
// **********************************************************************
// *                                                                    *
// * Code to perform communication with the I2C on the ICH Chip.        *
// *                                                                    *
// * Written by the IBM Director Development Team                       *
// * (C) Copyright IBM Corporation, 1998-2003                           *
// *                                                                    *
// *  This software may be used and distributed according to the terms  *
// *  of the GNU Public License, incorporated herein by reference.      *
// *                                                                    *
// **********************************************************************

#ifndef I2CICH_H
#define I2CICH_H

// ICH PCI function 3 registers
#define ICH_SM_REG_BASE_ADDR 0x20
#define ICH_SMBHSTCFG 0x40       //  SMBus Configuration register
#define ICH_SMB_INUSE 0x40   
#define ICH_SMB_HOSTBUSY 0x01
#define ICH_INT_ENABLE 0x02 
#define ICHSMBINTRPN 0x3d
#define ICHENABLE 0
#define ICHDISABLE 1  
#define ICH_MAX_DELAY 10000
#define ICH_DELAY_TIME 100

//  ICH SMBus register offsets from base address
//   same as for PIIX4.  Defined in i2cpiix4.h and
//   included here for reference only

#define ICH_HST_STA   0x00       //  host status register
#define ICH_HST_CNT   0x02       //  host control register
#define ICH_HST_CMD   0x03       //  host command register
#define ICH_XMIT_SLVA 0x04       //  host address register
#define ICH_HST_D0    0x05      //  host data 0 register
#define ICH_HST_D1    0x06      //  host data 1 register     
#define ICH_BLOCK_DB  0x07

#define ICH_START 0x40

// Host Status Register bit definitions
        
#define ICH_SMB_CMD_BLOCK 0x14
#define ICH_SMB_CMD_BYTE  0x04        
 

#ifdef SMBALERT
//  ICH macros
// 
#define ICH_CLEAR_HOST_STATUS() OUTP( (SMBusBaseAddr + ICH_HST_STA), 0x7e)
#else
#define ICH_CLEAR_HOST_STATUS() OUTP( (SMBusBaseAddr + ICH_HST_STA), 0x5e)
#endif

// Exported functions
//
I2CSTATUS I2CICHPointerSetThenRead2(BYTE slave_addr, BYTE command_code, void FAR *byte_read,
                                      int number_bytes_read, int number_retries);
I2CSTATUS I2CICHRead2(BYTE slave_addr, void FAR *byte_read, int number_bytes_read, int number_retries);
I2CSTATUS I2CICHPointerSetThenWrite2(BYTE slave_addr, BYTE command_code, void FAR *bytes_written,
                                       int number_bytes_written, int number_retries);
I2CSTATUS I2CICHPointerSetThenReadBlock2(BYTE slave_addr, BYTE command_code, BYTE FAR *byte_read,
                                    int *number_bytes_read, int number_retries);
I2CSTATUS I2CICHPointerSetThenWriteBlock2(BYTE slave_addr, BYTE command_code, BYTE FAR *bytes_written,
                                     int number_bytes_written, int number_retries);
I2CSTATUS I2CICHPointerSet2(BYTE IN address, BYTE IN command);
  

#endif  // I2CICH_H
