
// **********************************************************************
// *                                                                    *
// * This is a common code file for the SMBUS I/O.                       *
// *                                                                    *
// * Written by the IBM Director Development Team                       *
// * (C) Copyright IBM Corporation, 1998-2003                           *
// *                                                                    *
// *  This software may be used and distributed according to the terms  *
// *  of the GNU Public License, incorporated herein by reference.      *
// *                                                                    *
// ********************************************************************** 
 
#ifndef __KERNEL__
#  define __KERNEL__
#endif
#ifndef MODULE
#  define MODULE
#endif

#define __NO_VERSION__
#include <linux/module.h> /* get MOD_DEC_USE_COUNT, not the version string */
#include <linux/version.h> /* need it for conditionals */

#include <linux/kernel.h> /* printk() */
// #include <linux/malloc.h> /* kmalloc() */
#include <linux/fs.h>     /* everything... */
#include <linux/errno.h>  /* error codes */
#include <linux/types.h>  /* size_t */
#include <linux/fcntl.h>
#include <linux/tty.h>    /* current->tty */

/*
// #include "i2cos.h"  FH3: this will be included below via i2cio.h   FxFx: Remove these includes
#include "i2cio.h"
#include "i2cpiix4.h"
#include "i2cvia.h"
#include "i2csis.h"
#include "i2cich.h"
// #include "i2cgpio.h"   FH3: drop GPIO support
#include "pci.h"
*/

#include "i2cos.h"
#include "i2cio.h"
#include "i2cali.h"
#include "i2cpiix4.h"
#include "i2cvia.h"
#include "i2csis.h"
#include "pci.h"
#include "i2cich.h"
// #include <string.h>  FH3
// #include <stdio.h>   FH3

int MAX_RETRIES = 8;

#ifdef __OS2__
static int port_base;
#else
static void FAR *port_base;
#endif

unsigned int SMBusBaseAddr = 0;
int IsALIFound = 0;
int IsPIIX4Found = 0;
int IsOSB4Found = 0;
int IsCSB5Found = 0;
int IsForceGPIO = 0;
int IsGPIOFound = 0;
int IsICHFound = 0;
int IsICH2Found = 0;
int IsVIAFound = 0;
int IsVIA2Found = 0;
int IsSISFound = 0;
int disableSMBus = 0;
//int TestGPIO();
//int TestRanger();

//
// CSB locations for doing a host reset.
int CSB5bus = 0;
int CSB5device = 0;
unsigned long CSB5config;
unsigned long CSB5address;
unsigned long CSB5control;

BYTE DATA_HIGH = DATA_HIGH_DEFAULT;
BYTE CLOCK_HIGH = CLOCK_HIGH_DEFAULT;


// -------------------
// Internal Function Prototypes
// -------------------
I2CSTATUS I2CPointerSetThenRead2(BYTE addr, 
                                 BYTE pointer, 
                                 void FAR *retval, 
                                 int length, 
                                 int retry);
I2CSTATUS I2CRead2(BYTE addr, void FAR *retval, int length, int retry);
I2CSTATUS I2CPointerSetThenWrite2(BYTE addr, BYTE pointer, 
                                  void FAR *retval, int length, int retry);


#ifndef WIN32
void I2CSetRetries(int num_retries)
{
   MAX_RETRIES = num_retries;
}
#endif

// Initialize GPIO 20 and 21 (the I2C SDA and SCL)
#ifdef __OS2__
void I2CInit(int port_base_address)
#else
void I2CInit(void FAR *port_base_address)
#endif
{
   BYTE data, dir;
   int bus, device;

   printk("Start I2CInit..\n");
	
// Fix for defect 11264 -- prevent this driver from accessing the SMBus when
// a service processor is present.
   if (PCIWalk(IBM, WISEMAN, &bus, &device))
   {
   // Detected a Wiseman
      disableSMBus = 1;
   }
   else if (PCIWalk(IBM, CONDOR, &bus, &device))
   {
   // Detected a Eagle, Condor, or Falcon
      disableSMBus = 1;
   }
/* FxFx: Add Ranger support later   
   else if (TestRanger())
   {
   // Detected a Ranger
      disableSMBus = 1;
   }
*/

   if ((!disableSMBus) && (PCIWalk(INTEL, PIIX4, &bus, &device)) )
   {
      IsPIIX4Found = 1;
   }
   else if ((!disableSMBus) && (PCIWalk(INTEL, I443MX, &bus, &device)) )
   {
      IsPIIX4Found = 1;
   }
   else if ((!disableSMBus) && (PCIWalk(INTEL, ICH, &bus, &device)) )
   {
      IsICHFound = 1;
   }
   else if ((!disableSMBus) && (PCIWalk(INTEL, ICH2, &bus, &device)) )
   {
      IsICH2Found = 1;
   }
   else if ((!disableSMBus) && (PCIWalk(INTEL, ICH2M, &bus, &device)) )
   {
      IsICH2Found = 1;
   }
   else if ((!disableSMBus) && (PCIWalk(INTEL, ICH3, &bus, &device)) )
   {
      IsICH2Found = 1;
   }
   else if ((!disableSMBus) && (PCIWalk(INTEL, ICH4, &bus, &device)) )
   {
      IsICH2Found = 1;
   }
   else if ((!disableSMBus) && (PCIWalk(INTEL, ICH5, &bus, &device)) )
   {
      IsICH2Found = 1;
   }
   else if ((!disableSMBus) && (PCIWalk(INTEL, ICHS, &bus, &device)) )
   {
      IsICH2Found = 1;
   }
   else if ((!disableSMBus) && (PCIWalk(VIA, PIPC, &bus, &device)) )
   {
      IsVIAFound = 1;
   }
   else if ((!disableSMBus) && (PCIWalk(VIA, V686, &bus, &device)) )
   {
      IsVIA2Found = 1;
   }
   else if ((!disableSMBus) && (PCIWalk(SIS, SISSMB, &bus, &device)) )
   {
      IsSISFound = 1;
   }
   else if ((!disableSMBus) && (PCIWalk(RELIANCE, OSB4, &bus, &device)) )
   {
      IsOSB4Found = 1;
   }
   else if ((!disableSMBus) && (PCIWalk(RELIANCE, CSB5, &bus, &device)) )
   {
      IsCSB5Found = 1;
   }
   // d37424: Map to osb4 for 4.1.1
   else if ((!disableSMBus) && (PCIWalk(RELIANCE, CSB6U, &bus, &device)) )
   {
      IsOSB4Found = 1;
   }   
   else if ((!disableSMBus) && (PCIWalk(ALI, M1535, &bus, &device)) )
   {
      IsALIFound = 1;
   }
   else if (!disableSMBus)
   {
      // IsGPIOFound = TestGPIO();   FxFx: Add GPIO support later ??
   }

   if ((IsICHFound) && (! IsForceGPIO))
   {
      unsigned long pcireg;

      //  DbgPrint("ICH Device Found");  FH3
      printk("ICH Device Found.\n");

//      pcireg = PCI_GetConfigRegister(bus, device, 3, (PCICMD>>2) );
//      pcireg |= 1;
//      PCI_SetConfigRegister(bus, device, 3, (PCICMD>>2), pcireg);

   // Enable SMBus controller
      pcireg = PCI_GetConfigRegister(bus, device, 3, (ICH_SMBHSTCFG>>2));
      pcireg |= ((unsigned long)ICHSMB_ENABLE);
      PCI_SetConfigRegister(bus, device, 3, (ICH_SMBHSTCFG>>2), pcireg);

   // Mask off the low 4 bits and save the Base Address
      SMBusBaseAddr = (0xfff0 & (unsigned int)PCI_GetConfigRegister(
         bus, device, 3, (ICH_SM_REG_BASE_ADDR>>2) ));

//      ICH_CLEAR_HOST_STATUS();
   }
   else if ((IsICH2Found) && (! IsForceGPIO))
   {
      unsigned long pcireg;

      // DbgPrint("ICHx Device Found");   FH3
      printk("ICHx Device Found.\n");

//      pcireg = PCI_GetConfigRegister(bus, device, 3, (PCICMD>>2) );
//      pcireg |= 1;
//      PCI_SetConfigRegister(bus, device, 3, (PCICMD>>2), pcireg);

   // Enable SMBus controller
      pcireg = PCI_GetConfigRegister(bus, device, 3, (ICH_SMBHSTCFG>>2));
      pcireg |= ((unsigned long)ICHSMB_ENABLE);
      PCI_SetConfigRegister(bus, device, 3, (ICH_SMBHSTCFG>>2), pcireg);

   // Mask off the low 4 bits and save the Base Address
      SMBusBaseAddr = (0xfff0 & (unsigned int)PCI_GetConfigRegister(bus, device, 3, (ICH_SM_REG_BASE_ADDR>>2) ));

   // next section added 11/10/00 defect 9902 - In Win2K, the ICH2
   // SMBus device was detected by the PnP enumeration, and the device was
   // disabled by Win2K (resulting in the i/o enable bit being set to 0).
   // Note that this is only an issue for ICH2 (this bit is hardwired to
   // a 1 in ICH).
      pcireg = PCI_GetConfigRegister(bus, device, 3, (0x04>>2));
      pcireg |= ((unsigned long)1);
      PCI_SetConfigRegister(bus, device, 3, (0x04>>2), pcireg);

      ICH_CLEAR_HOST_STATUS();
   }
   else if ((IsVIAFound) && (! IsForceGPIO))
   {
     unsigned long pcireg;
     // DbgPrint("VIA Device Found");  FH3
     printk("VIA Device Found.\n");

     pcireg = PCI_GetConfigRegister(bus, device, 3, (VIA_PCICMD>>2) );
     pcireg |= 1;
     PCI_SetConfigRegister(bus, device, 3, (VIA_PCICMD>>2), pcireg);

  // Enable SMBus controller
     pcireg = PCI_GetConfigRegister(bus, device, 3, (VIA_SMBHSTCFG>>2));
     pcireg |= ((unsigned long)VIASMB_ENABLE << 16);
     PCI_SetConfigRegister(bus, device, 3, (VIA_SMBHSTCFG>>2), pcireg);

  // Mask off the low 4 bits and save the Base Address
     SMBusBaseAddr = (0xfff0 & (unsigned int)PCI_GetConfigRegister(
        bus, device, 3, (VIA_SM_REG_BASE_ADDR>>2) ));
       
     VIACLEAR_HOST_STATUS();
   }
   else if ((IsVIA2Found) && (! IsForceGPIO))
   {  
     unsigned long pcireg;
     // DbgPrint("VIA2 Device Found");  FH3
     printk("VIA2 Device Found.\n");

     pcireg = PCI_GetConfigRegister(bus, device, 4, (VIA_PCICMD>>2) );
     pcireg |= 1;
     PCI_SetConfigRegister(bus, device, 4, (VIA_PCICMD>>2), pcireg);

  // Enable SMBus controller
     pcireg = PCI_GetConfigRegister(bus, device, 4, (VIA_SMBHSTCFG>>2));
     pcireg |= ((unsigned long)VIASMB_ENABLE << 16);
     PCI_SetConfigRegister(bus, device, 4, (VIA_SMBHSTCFG>>2), pcireg);

  // Mask off the low 4 bits and save the Base Address
     SMBusBaseAddr = (0xfff0 & (unsigned int)PCI_GetConfigRegister(
        bus, device, 4, (VIA_SM_REG_BASE_ADDR>>2) ));
       
  // Via2 seems to be register compatible with the Via, so use that code
     IsVIAFound = 1;

     VIACLEAR_HOST_STATUS();
   }
   else if ((IsSISFound) && (! IsForceGPIO))
   {  
     //  DbgPrint("SIS Device Found");  FH3
     printk("SIS Device Found.\n");

  // Mask off the low 4 bits and save the Base Address
     SMBusBaseAddr = ((unsigned int) PCI_GetConfigRegister(bus, device, 0, SIS_SM_REG_BASE_ADDR>>2)) & 0xffe0;

  // Disable Interrupts
     OUTP( SMBusBaseAddr + SIS_SMB_EN, 0x00);

  // Configure Host
     OUTP( SMBusBaseAddr + SIS_SMB_CNT, 0xE0);
       
     SIS_CLEAR_HOST_STATUS();
   }
   else if ((IsPIIX4Found) && (! IsForceGPIO))
   {
     unsigned long pcireg;

     // DbgPrint("PIIX4 Device Found");  FH3
     printk("PIIX4 Device Found.\n");
 
     pcireg = PCI_GetConfigRegister(bus, device, 3, (PIIX4_PCICMD>>2) );
     pcireg |= 1;
     PCI_SetConfigRegister(bus, device, 3, (PIIX4_PCICMD>>2), pcireg);

  // Enable SMBus controller
     pcireg = PCI_GetConfigRegister(bus, device, 3, (PIIX4_SMBHSTCFG>>2));
     pcireg |= ((unsigned long)PIIX4SMB_ENABLE << 16);
     PCI_SetConfigRegister(bus, device, 3, (PIIX4_SMBHSTCFG>>2), pcireg);

  // Mask off the low 4 bits and save the Base Address
     SMBusBaseAddr = (0xfff0 & (unsigned int)PCI_GetConfigRegister(
        bus, device, 3, (PIIX4_SM_REG_BASE_ADDR>>2) ));
    
     PIIX4_CLEAR_HOST_STATUS ();
  }
  if ((IsOSB4Found || IsCSB5Found)) //&& (! IsForceGPIO))
  {
     unsigned long pcireg;

     // DbgPrint("OSB4 Device Found");  FH3
     if (IsCSB5Found) 
          printk("CSB5 Device Found.\n");
     else
          printk("OSB4 Device Found.\n");

//     pcireg = PCI_GetConfigRegister(bus, device, 0, (PCICMD>>2) );
//     pcireg |= 1;
//     PCI_SetConfigRegister(bus, device, 0, (PCICMD>>2), pcireg);

  // Enable SMBus controller
  // Removed this code 8/8/00 
  // The SMBus driver will
  // not enable the SMBus support in the OSB4 if it is not
  // already enabled.  The BIOS will only enable the support
  // if no Service Processor is present in the system.  If the
  // service processor is present, then the service processor
  // (not this driver) controls the SMBus.
  // However, this BIOS change was not made until after SeaHero -- on
  // SeaHero, we must always manually enable the SMBus.  The test
  // ForceEnableOSB4() tells us whether or not the system is a SeaHero
  // and needs to be enabled by this driver.

  // 12/6/00 -- Removed the test for Sea Hero, since this is now
  // handled by the test for presence of a Wiseman above.
//     if (ForceEnableOSB4())
//     {
        pcireg = PCI_GetConfigRegister(bus, device, 0, (PIIX4_SMBHSTCFG>>2));
        pcireg |= ((unsigned long)PIIX4SMB_ENABLE << 16);
        PCI_SetConfigRegister(bus, device, 0, (PIIX4_SMBHSTCFG>>2), pcireg);
//     }

	//
	// Added for the host reset.
	CSB5bus = bus;
	CSB5device = device;
    CSB5config = PCI_GetConfigRegister(CSB5bus, CSB5device, 0, (PIIX4_SMBHSTCFG>>2));
    CSB5address = PCI_GetConfigRegister(CSB5bus, CSB5device, 0, (PIIX4_SM_REG_BASE_ADDR>>2));
    CSB5control = PCI_GetConfigRegister(CSB5bus, CSB5device, 0, (CSB5_PCICONTROL>>2));

  // Enable OSB4 CMS decodes and functions
//     pcireg = PCI_GetConfigRegister(bus, device, 0, (OSB4_MISC_REGISTER>>2));
//     pcireg |= ((unsigned long)OSB4_CMS_Enable << 8);
//     PCI_SetConfigRegister(bus, device, 0, (OSB4_MISC_REGISTER>>2), pcireg);

  // Mask off the low 4 bits and save the Base Address
     SMBusBaseAddr = (0xfff0 & (unsigned int)PCI_GetConfigRegister(
                      bus, device, 0, (PIIX4_SM_REG_BASE_ADDR>>2) ));

     PIIX4_CLEAR_HOST_STATUS ();
  }
  else if ((IsALIFound) && (! IsForceGPIO))
  {
    unsigned long pcireg;

    // DbgPrint("ALI (M1535) Chipset Found");  FH3
    printk("ALI (M1535) Chipset Found.\n");

    // Enable SMbus controller
    pcireg = PCI_GetConfigRegister (bus, device, 0, (ALI_SMBHSTCFG >> 2));
    pcireg |= ALISMB_ENABLE;

    PCI_SetConfigRegister (bus, device, 0, (ALI_SMBHSTCFG >> 2), pcireg);

    // Mask off the low 4 bits and save the Base Address
    SMBusBaseAddr = (0xfff0 & ((unsigned int)PCI_GetConfigRegister (bus, device, 0, (ALI_SM_REG_BASE_ADDR >> 2))) >> 16);

    // Clear all SMBus host controller status bits
    OUTP (SMBusBaseAddr + ALI_SMB_STA, INP (SMBusBaseAddr + ALI_SMB_STA));
 }
  else if (IsGPIOFound || IsForceGPIO)
  {
  // Initialize Global variable
     port_base = port_base_address;

  // Set clock and data high (inactive state)
     data = INP(GPIO2DATA);
     data |= 3;
     OUTP(GPIO2DATA, data);
 
  // Note -- the driver should not do this so that it can remain generic.
  // The output-type should be setup by the BIOS.
  // Set clock and data to open-drain   
  //   out = INP(GPIO2OUTTYPE);
  //   out &= 0xfc;
  //   OUTP(GPIO2OUTTYPE, out);
   
  // Set direction of clock and data to output
     dir = INP(GPIO2DIR);
     dir |= 3;
     OUTP(GPIO2DIR, dir);
   
  // Note -- the driver should not do this so that it can remain generic.
  // The pullup-type should be setup by the BIOS.
  // Set clock and data to no pull-up
  //   pull = INP(GPIO2PULLUP);
  //   pull &= 0xfc;
  //   OUTP(GPIO2PULLUP, pull);
  }
}

/*   --- FxFx: add the following functions one at a time
// Simulate Start Condition with a HIGH to LOW transition of SDA 
// while clock held high.
void I2CStart()
{
   BYTE data;

   data = INP(GPIO2DATA);
   data |= DATA_HIGH;
   OUTP(GPIO2DATA, data);

// Data in setup time to clock high 100ns
   DELAY(SETUP_TIME); // 100 nanoseconds
   
   data = INP(GPIO2DATA);
   data |= CLOCK_HIGH;
   OUTP(GPIO2DATA, data);

// Wait 1/4 of clock period
   DELAY(PERIOD/4);   // 625 nanoseconds
   
   data = INP(GPIO2DATA);
   data &= ~DATA_HIGH;
   OUTP(GPIO2DATA, data);

// data low setup time to clock low must be >= 100ns (start condition)     
   DELAY(PERIOD/4);   // 625 nanoseconds
   
   data = INP(GPIO2DATA);
   data &= ~CLOCK_HIGH;
   OUTP(GPIO2DATA, data);
   
   DELAY(PERIOD/2);   // 1.25 Microseconds
}


// Simulate Stop Condition with a LOW to HIGH transition of SDA 
// while clock held high.
void I2CStop()
{
   BYTE data;
   
   data = INP(GPIO2DATA);
   data &= ~DATA_HIGH;
   OUTP(GPIO2DATA, data);

// Data in setup time to clock high 100ns
   DELAY(SETUP_TIME); // 100 nanoseconds
   
   data = INP(GPIO2DATA);
   data |= CLOCK_HIGH;
   OUTP(GPIO2DATA, data);

// Wait 1/4 of clock period
   DELAY(PERIOD/4);   // 625 nanoseconds
   
   data = INP(GPIO2DATA);
   data |= DATA_HIGH;
   OUTP(GPIO2DATA, data);

// data high hold time after clock high must be >= 100ns (stop condition)
   DELAY(PERIOD/4);   // 625 nanoseconds

// Should leave clock high after stop condition   
//   data = INP(GPIO2DATA);
//   data &= ~CLOCK_HIGH;
//   OUTP(GPIO2DATA, data);
//   
//// Clock period is 2.5 us
//   DELAY(PERIOD/2);   // 1.25 Microseconds
}


// Sends a single clock cycle to the I2C with the data line high while
// the clock is high.  
// The function exits with the clock low, and guarantees that enough
// time has expired so that the next function can immediately take 
// the clock high.
void I2CSendHiBit()
{
   BYTE data;
   
   data = INP(GPIO2DATA);
   data |= DATA_HIGH;
   OUTP(GPIO2DATA, data);
   
// Data in setup time to clock high 100ns
   DELAY(SETUP_TIME); // 100 nanoseconds

   data = INP(GPIO2DATA);
   data |= CLOCK_HIGH;
   OUTP(GPIO2DATA, data);
   
   DELAY(PERIOD/2);   // 1.25 Microseconds
   
   data = INP(GPIO2DATA);
   data &= ~CLOCK_HIGH;
   OUTP(GPIO2DATA, data);
   
   DELAY(PERIOD/2);   // 1.25 Microseconds
}


// Sends a single clock cycle to the I2C with the data line low while
// the clock is high.  
// The function exits with the clock low, and guarantees that enough
// time has expired so that the next function can immediately take 
// the clock high.
void I2CSendLoBit()
{
   BYTE data;
   
   data = INP(GPIO2DATA);
   data &= ~DATA_HIGH;
   OUTP(GPIO2DATA, data);
   
// Data in setup time to clock high 100ns
   DELAY(SETUP_TIME); // 100 nanoseconds

   data = INP(GPIO2DATA);
   data |= CLOCK_HIGH;
   OUTP(GPIO2DATA, data);
   
   DELAY(PERIOD/2);   // 1.25 Microseconds
   
   data = INP(GPIO2DATA);
   data &= ~CLOCK_HIGH;
   OUTP(GPIO2DATA, data);
   
   DELAY(PERIOD/2);   // 1.25 Microseconds
}


// Send an ACK to the I2C device by pulling the data line low while the
// device has released it high.
void I2CSendACK()
{
   BYTE data, save;

// Set direction of clock and data to point OUT 
//  (bit 0 (data) = 1, bit 1 (clock) = 1)  
   save = data = INP(GPIO2DIR);
   data |= (CLOCK_HIGH | DATA_HIGH);
   OUTP(GPIO2DIR, data);

   I2CSendLoBit();

// Restore direction
   INP(GPIO2DIR);   // Always do a read-modify-write to circumvent 308 bug
   OUTP(GPIO2DIR, save);
}


// Indicate a NAK by releasing the data line high for one clock cycle
void I2CSendNAK()
{
   I2CSendHiBit();
}


// Returns ERROR_NO_ERROR if the I2C device sends an ACK during the
// clock cycle, or ERROR_NO_ACK if an ACK is not seen.  
// To wait for an ACK, the function should release the data line high, 
// raise the clock high, and see whether the device pulls the data
// line low WHILE THE CLOCK IS HIGH.  The device is free to release
// the data line as soon as the clock transitions low.
I2CSTATUS I2CWaitForAck()
{
   BYTE data, save, ack;

// Send data high to receive ACK
   data = INP(GPIO2DATA);
   data |= DATA_HIGH;
   OUTP(GPIO2DATA, data);
   
// Set direction of data IN (leave clock pointing OUT)
   save = data = INP(GPIO2DIR);
   data &= ~DATA_HIGH;
   OUTP(GPIO2DIR, data);

// Data in setup time to clock high 100ns
   DELAY(SETUP_TIME); // 100 nanoseconds
   
// Send clock high
   data = INP(GPIO2DATA);
   data |= CLOCK_HIGH;
   OUTP(GPIO2DATA, data);
   
   DELAY(PERIOD/4);   // 625 nanoseconds

// ack contains the data from the I2C in the LSBit
   ack = INP(GPIO2DATA);
   
   DELAY(PERIOD/4);   // 625 nanoseconds

// Send clock low   
   data = INP(GPIO2DATA);
   data &= ~CLOCK_HIGH;
   OUTP(GPIO2DATA, data);

   DELAY(PERIOD/2);   // 1.25 Microseconds

// Restore direction
   INP(GPIO2DIR);     // 308 bug
   OUTP(GPIO2DIR, save);

   if (ack & DATA_HIGH)
      return ERR_NO_ACK;
   else
      return ERR_NO_ERROR;
}


// Sends an 8 bit value on the I2C bus
void I2CSendByte(BYTE value)
{
   static BYTE mask[8] = {0x01, 0x02, 0x04, 0x08, 0x10, 0x20, 0x40, 0x80};
   int i;
   
   for (i=7; i>=0; i--)
   {
      if (value & mask[i])
         I2CSendHiBit();
      else
         I2CSendLoBit();
   }
}

                     
// Reads an 8 bit value on the I2C bus.  Each bit is read by raising the
// clock, waiting for 1/4 clock period (half the time that the clock is high)
// then reading the data line.
void I2CReadByte(BYTE FAR *value)
{
   static BYTE mask[8] = {0x01, 0x02, 0x04, 0x08, 0x10, 0x20, 0x40, 0x80};
   BYTE data;
   int i;

   *value = (BYTE)0;   
   for (i=7; i>=0; i--)
   {
   // Send clock high
      data = INP(GPIO2DATA);
      data |= CLOCK_HIGH;
      OUTP(GPIO2DATA, data);
   
      DELAY(PERIOD/4);   // 625 nanoseconds

   // data contains the data from the I2C in the LSBit
      data = INP(GPIO2DATA);
      if (data & DATA_HIGH)
         (*value) |= mask[i];

      DELAY(PERIOD/4);   // 625 nanoseconds

   // Send clock low   
      data = INP(GPIO2DATA);
      data &= ~CLOCK_HIGH;
      OUTP(GPIO2DATA, data);
   
      DELAY(PERIOD/2);   // 1.25 Microseconds
   }
}


I2CSTATUS I2CPointerSetThenRead2(BYTE addr, BYTE pointer, 
                                  void FAR *retval, int length, int retry)
{
   BYTE data;
   I2CSTATUS status;

// ----------------------------------
//  Write (send address and pointer)
// ----------------------------------

// Set direction of clock and data to point OUT (bit 0 (data) = 1, bit 1 (clock) = 1)  
   data = INP(GPIO2DIR);
   data |= (CLOCK_HIGH | DATA_HIGH);
   OUTP(GPIO2DIR, data);
   
   I2CStart();

// Send Address
   I2CSendByte((BYTE)( (addr & 0xFE) + WRITE));
   status = I2CWaitForAck();    
   if (status == ERR_NO_ACK)
   {
      // DbgPrint("No ACK received from I2C after addr\n");  FH3
      printk("No ACK received from I2C after addr\n");
      I2CStop();
      if (retry >= MAX_RETRIES)
         return status;
      else
      {
         DELAY(10000);
         return I2CPointerSetThenRead2(addr, pointer, retval, length, retry+1);
      }
   }

// Send Pointer
   I2CSendByte(pointer);
   status = I2CWaitForAck();    
   if (status == ERR_NO_ACK)
   {
      // DbgPrint("No ACK received from I2C after pointer\n");   FH3
      printk("No ACK received from I2C after pointer\n");
      I2CStop();
      if (retry >= MAX_RETRIES)
         return status;
      else
      {
         DELAY(10000);
         return I2CPointerSetThenRead2(addr, pointer, retval, length, retry+1);
      }
   }
   
// Restart and perform a normal read
   return I2CRead2(addr, retval, length, retry);
}


I2CSTATUS I2CRead2(BYTE addr, void FAR *retval, int length, int retry)
{
   BYTE data;
   BYTE value;
   I2CSTATUS status;
   int i;

// Initialize retval
   for (i=0; i<length; i++)
      ((char FAR*)retval)[i] = 0;
   
   I2CStart();
   
// Send Address
   I2CSendByte((BYTE)( (addr & 0xFE) + READ));
   status = I2CWaitForAck();    
   if (status == ERR_NO_ACK)
   {
      // DbgPrint("No ACK received from I2C after addr\n");  FH3
      printk("No ACK received from I2C after addr\n");
      I2CStop();
      if (retry >= MAX_RETRIES)
         return status;
      else
      {
         DELAY(10000);
         return I2CRead2(addr, retval, length, retry+1);
      }
   }

// -----------
//  Read data
// -----------

// Set direction of data IN (leave clock pointing OUT)
   data = INP(GPIO2DIR);
   data &= ~DATA_HIGH;
   OUTP(GPIO2DIR, data);

// Read the data from the I2C bus, sending an ACK after each byte 
// (except the last).
// The bytes are accumulated in Big Endian order.
   for (i=0; i<length-1; i++)
   {
      I2CReadByte(&value);
      ((char FAR*)retval)[i] = value;
      I2CSendACK();
   }
// Special case -- send NAK after reading last byte   
   I2CReadByte(&value);
   ((char FAR*)retval)[i] = value;

// Convert from Big Endian (network order) to Little Endian (host order), 
// if necessary   
//   if (length == 2)
//   {  
//      i = *((int FAR*)retval);
//      *((unsigned short FAR*)retval) = ntohs((unsigned short)i);
//   }

// ----------------------------------
//  Write (send NAK and Stop)
// ----------------------------------
// Set direction of clock and data to point OUT (bit 0 (data)=1,bit 1 (clock)=1)  
   data = INP(GPIO2DIR);
   data |= (CLOCK_HIGH | DATA_HIGH);
   OUTP(GPIO2DIR, data);
   
   I2CSendNAK();
   
   I2CStop();
   
   return ERR_NO_ERROR;
}


I2CSTATUS I2CPointerSetThenWrite2(BYTE addr, BYTE pointer, 
                                   void FAR *value, int length, int retry)
{
   BYTE data;
   I2CSTATUS status;
   int i;

// ----------------------------------
//  Write (send address and pointer)
// ----------------------------------

// Set direction of clock and data to point OUT 
//  (bit 0 (data) = 1, bit 1 (clock) = 1)  
   data = INP(GPIO2DIR);
   data |= (CLOCK_HIGH | DATA_HIGH);
   OUTP(GPIO2DIR, data);
   
   I2CStart();

// Send Address
   I2CSendByte((BYTE)( (addr & 0xFE) + WRITE));
   status = I2CWaitForAck();    
   if (status == ERR_NO_ACK)
   {
      // DbgPrint("No ACK received from I2C after addr\n");   FH3
      printk("No ACK received from I2C after addr\n");
      I2CStop();
      if (retry >= MAX_RETRIES)
         return status;
      else
      {
         DELAY(10000);
         return I2CPointerSetThenWrite2(addr, pointer, value, length, retry+1);
      }
   }

// Send Pointer
   I2CSendByte(pointer);
   status = I2CWaitForAck();    
   if (status == ERR_NO_ACK)
   {
      // DbgPrint("No ACK received from I2C after pointer\n");  FH3
      printk("No ACK received from I2C after pointer\n");
      I2CStop();
      if (retry >= MAX_RETRIES)
         return status;
      else
      {
         DELAY(10000);
         return I2CPointerSetThenWrite2(addr, pointer, value, length, retry+1);
      }
   }

// -----------
//  Write data
// -----------

// Read the data from the I2C bus, sending an ACK after each byte 
// (except the last).
// The bytes are sent LSB first.
   for (i=0; i<length; i++)
   {
      I2CSendByte( ((char FAR*)value)[i] );
      status = I2CWaitForAck();    
      if (status == ERR_NO_ACK)
      {
         // DbgPrint("No ACK received from I2C after byte i\n");  FH3
         printk("No ACK received from I2C after byte i\n");
         I2CStop();
         if (retry >= MAX_RETRIES)
            return status;
         else
         {
            DELAY(10000);
            return I2CPointerSetThenWrite2(addr,pointer,value,length,retry+1);
         }
      }
   }
   
   I2CStop();
   
   return ERR_NO_ERROR;
}
*/

// ----------------------------------------------------------------------


// --------------------------
//    Exported Functions
// --------------------------

// Protocol legend:
//   S = start condition
//   a = slave address
//   r = read bit
//   w = write bit
//   c = command code
//   d = data 
//   A = ACK
//   N = NAK
//   P = stop condition
//   - = (underline) indicates a bit that is sent by the slave device.

// Exported function that implements the Read Byte/Word protocol for SMBus.
//
// slave_addr is the address of the device on the I2C bus.
// command_code is a device specific code 
//   For example, on the LM75 chip, the command code should be set to either
//   0,1,2, or 3 to read temperature, configuration, t_hyst, or t_os registers.
// length should either be 1 or 2, depending on whether the function is being
//    asked to read a 1 byte value or a 2 byte value (word).
//
// Protocol: S aaaaaaaw A cccccccc A S sssssssr A dddddddd N P            (byte)
//                      -          -            - --------
// or        S aaaaaaaw A cccccccc A S sssssssr A dddddddd A dddddddd N P (word)
//                      -          -            - --------   --------
// The function returns ERR_NO_ERROR on success.
I2CSTATUS I2CReadData(BYTE slave_addr, BYTE command_code, 
                      void FAR *retval, int length)
{
I2CSTATUS TStatus = 0;
   if(IsICHFound) 
   {
      return I2CICHPointerSetThenRead2(slave_addr, command_code, retval, length, 0);
   }
   else if(IsICH2Found) 
   {
	TStatus = I2CICHPointerSetThenRead2(slave_addr, command_code, retval, length, 0);
	debug_out(DETAILS_LVL,"I2CReadData.  slave_addr = %xh, command_code = %xh, retval = %xh, length = %dd.\n", slave_addr, command_code, (int*)retval, length);
	return (TStatus);
   }
   else if (IsVIAFound || IsVIA2Found)
   {  
       debug_out(DETAILS_LVL,"I2CReadData.  slave_addr = %xh, command_code = %xh, length = %xh.\n", slave_addr, command_code, length);
	   return I2CVIAPointerSetThenRead2(slave_addr, command_code, retval, length, 0);
   }
   else if (IsPIIX4Found || IsOSB4Found || IsCSB5Found)
   {  
		//next line changed (defect 12425) to add support for
		//more PIIX4 functions on the OSB4
		return PIIX4_SMBusRead (slave_addr, command_code, retval, length, 0);
      //return I2CPIIX4PointerSetThenRead2(slave_addr, command_code, retval, length, 0);
   }
   else if (IsSISFound)
   {  
      return I2CSISPointerSetThenRead2(slave_addr, command_code, retval, length, 0);
   }
   else if (IsALIFound)
   {  
      return ALI_SMBusRead(slave_addr, command_code, retval, length, 0);
   }
   else if (IsGPIOFound || IsForceGPIO)
   {
      // return I2CPointerSetThenRead2(slave_addr, command_code, retval, length, 0);
	  // FH3: Drop the above. GPIO not supported in linux port.
      debug_out(DETAILS_LVL,"I2CReadData.  IsGPIOFound || IsForceGPIO.\n");
   }

// Unknown SMBus device
   debug_out(DETAILS_LVL,"I2CReadData.  Unknown SMBus device. Return ERR_NOT_IMPLEMENTED.\n");
   return ERR_NOT_IMPLEMENTED;
}


// Exported function that implements the Write Byte/Word protocol for SMBus.
//
// Write the value in the 'length' LSBs of retval after sending the 
// command_code to the device.
//
// slave_addr is the address of the device on the I2C bus.
// command_code is a device specific code 
//   For example, on the LM75 chip, the command code should be set to
//   either 1,2, or 3 to write configuration, t_hyst, or t_os registers.
// The LSB(s) of value are written to the specified I2C register
// length should either be 1 or 2, depending on whether the function is being
//    asked to read a 1 byte value or a 2 byte value.                     
// 
// Protocol: S aaaaaaaw A cccccccc A dddddddd A P             (byte)
//                      -          -          -
// or        S aaaaaaaw A cccccccc A dddddddd A dddddddd A P  (word)
//                      -          -          -
// The function returns ERR_NO_ERROR on success.
I2CSTATUS I2CWriteData(BYTE slave_addr, BYTE command_code, 
                       void FAR *value, int length)
{  if (IsICHFound)
   {
      return I2CICHPointerSetThenWrite2(slave_addr, command_code, value, length, 0);
   }
   else if (IsICH2Found)
   {
	  debug_out(DETAILS_LVL,"I2CWriteData.  slave_addr = %xh, command_code = %xh, value = %xh, length = %xh.\n", slave_addr, command_code, (int*)value, length);
      return I2CICHPointerSetThenWrite2(slave_addr, command_code, value, length, 0);
   }
   else if (IsPIIX4Found || IsOSB4Found || IsCSB5Found)
   {
		//next line changed (defect 12425) to add support for
		//more PIIX4 functions on the OSB4
		return PIIX4_SMBusWrite (slave_addr, command_code, value, length, 0);
      //return I2CPIIX4PointerSetThenWrite2(slave_addr, command_code, value, length, 0);
   }
   else if (IsVIAFound || IsVIA2Found)
   {
	  debug_out(DETAILS_LVL,"I2CWriteData.  slave_addr = %xh, command_code = %xh, length = %xh.\n", slave_addr, command_code, length);
      return I2CVIAPointerSetThenWrite2(slave_addr, command_code, value, length, 0);
   }
   else if (IsSISFound)
   {
      return I2CSISPointerSetThenWrite2(slave_addr, command_code, value, length, 0);
   }
   else if (IsALIFound)
   {
      return ALI_SMBusWrite(slave_addr, command_code, value, length, 0);
   }
   else if (IsGPIOFound || IsForceGPIO)
   {
    //  return I2CPointerSetThenWrite2(slave_addr, command_code, value, length, 0);
   }

// Unknown SMBus device
   debug_out(DETAILS_LVL,"I2CWriteData.  Unknown SMBus device.  Return ERR_NOT_IMPLEMENTED.\n");
   return ERR_NOT_IMPLEMENTED;
}


// Exported function that implements the Receive Byte protocol for SMBus.
//
// Reads the value from the current internal address of the I2C device.
//
// slave_addr is the address of the device on the I2C bus.
// length indicates the number of successive bytes to read out of the device
//    (length == 1 is equivalent to the SMBus Receive Byte protocol).
// 
// Protocol: S aaaaaaar A (dddddddd A)* dddddddd N P
//                      -  --------     --------
// The function returns ERR_NO_ERROR on success.
I2CSTATUS I2CReceive(BYTE slave_addr, void FAR *retval, int length)
{  
   if(IsICHFound)
   {
      return I2CICHRead2(slave_addr, retval, length, 0);
   }
   else if(IsICH2Found)
   {
      debug_out(DETAILS_LVL,"I2CReceive. slave_addr = %xh.\n", slave_addr);
      return I2CICHRead2(slave_addr, retval, length, 0);
   }
   else if (IsPIIX4Found || IsOSB4Found || IsCSB5Found)
   {
		//next line added (defect 12425) to add support for
		//more PIIX4 functions on the OSB4
     return PIIX4_SMBusReceive (slave_addr, retval, length, 0);
   }
   else if (IsVIAFound || IsVIA2Found)
   {
	 debug_out(DETAILS_LVL,"I2CReceive.  slave_addr = %xh.\n", slave_addr);
     return I2CVIARead2(slave_addr, retval, length, 0);
   }
   else if (IsSISFound)
   {
     return I2CSISRead2(slave_addr, retval, length, 0);
   }
   else if (IsALIFound)
   {
     return ALI_SMBusReceive(slave_addr, retval, length, 0);
   }
   else if (IsGPIOFound || IsForceGPIO)
   {
    //  return I2CRead2(slave_addr, retval, length, 0);
   }

// Unknown SMBus device
   debug_out(DETAILS_LVL,"I2CReceive.  Unknown SMBus device. Return ERR_NOT_IMPLEMENTED.\n");
   return ERR_NOT_IMPLEMENTED;
}


// Exported function that implements the Send Byte protocol for SMBus.
//
// Writes the value to the current internal address of the I2C device.
//
// slave_addr is the address of the device on the I2C bus.
// length indicates the number of successive bytes to write
//    (length == 1 is equivalent to the SMBus Send Byte protocol).
// 
// Protocol: S aaaaaaaw A (dddddddd A)+ P
//                      -           -   
// The function returns ERR_NO_ERROR on success.
I2CSTATUS I2CSend(BYTE slave_addr, void FAR *retval, int length)
{  
 int i,status = 0;   // FH3
//KdPrint( ("I2CNT.SYS: I2CSEND HERE\n") );
  	
  debug_out(DETAILS_LVL,"I2CSend.  Start.\n");
	
  if ((IsICHFound) || (IsICH2Found))
   {
//KdPrint( ("I2CNT.SYS: IsICHFound || IsICH2Found\n") );
      for (i=0; i<length; i++)
      {
         status = I2CICHPointerSet2(slave_addr, ((BYTE*)retval)[i]);
         debug_out(DETAILS_LVL,"I2CSend.  status = %d\n", status);
         if (status != ERR_NO_ERROR)
         {
            return status;
         }
      }

      return ERR_NO_ERROR;
   }
   else if (IsVIAFound || IsVIA2Found)
   {

//KdPrint( ("I2CNT.SYS: IsVIAFound\n") );
      for (i=0; i<length; i++)
      {
           status = I2CVIAPointerSet2(slave_addr, ((BYTE*)retval)[i]);
         if (status != ERR_NO_ERROR)
         {
            return status;
         }
      }

      return ERR_NO_ERROR;
   }
   else if (IsSISFound)
   {
// KdPrint( ("I2CNT.SYS: IsSISFound\n") );
     return ERR_NOT_IMPLEMENTED;
   }
   else if (IsPIIX4Found || IsOSB4Found || IsCSB5Found)
   {
//KdPrint( ("I2CNT.SYS: IsPIIX4Found\n") );
//       return I2CPIIXSend2(slave_addr, retval, length, 0);
		//next section changed (defect 12425) to add support for
		//more PIIX4 functions on the OSB4
	   
      debug_out(DETAILS_LVL,"I2CSend  PIIX4Found.\n");
      for (i = 0; i < length; i++)
         {
          status = PIIX4_SMBusSend (slave_addr, ((BYTE*)retval)[i]);
          if (status != ERR_NO_ERROR)
              return (status);
         }

     return (ERR_NO_ERROR);
     //return ERR_NOT_IMPLEMENTED;
   }
   else if (IsALIFound)
   {
//KdPrint( ("I2CNT.SYS: IsALIFound\n") );
      for (i = 0; i < length; i++)
         {
            status = ALI_SMBusSend (slave_addr, ((BYTE*)retval)[i]);
          if (status != ERR_NO_ERROR)
              return (status);
         }

     return (ERR_NO_ERROR);
     //return ERR_NOT_IMPLEMENTED;
   }
   else if (IsGPIOFound || IsForceGPIO)
   {
      return ERR_NOT_IMPLEMENTED;
   }

//KdPrint( ("I2CNT.SYS: ERR DEFAULT\n") );
   return ERR_NOT_IMPLEMENTED;
}



// Exported function that implements the Receive Block protocol for SMBus.
//
// Reads the value from the current internal address of the I2C device.
//
// slave_addr is the address of the device on the I2C bus.
// length indicates the number of successive bytes to read out of the devica
//
// The function returns ERR_NO_ERROR on success.
I2CSTATUS I2CReadDataBlock(BYTE slave_addr, BYTE command_code,
                           BYTE *retval, int *length)
{  
   if(IsICHFound)
   {
      return I2CICHPointerSetThenReadBlock2(slave_addr, command_code, retval, length, 0);
   }
   else if (IsICH2Found)
   {
       debug_out(DETAILS_LVL,"I2CReadDataBlock.  slave_addr = %xh, command_code = %xh.\n", slave_addr, command_code);
      return I2CICHPointerSetThenReadBlock2(slave_addr, command_code, retval, length, 0);
   }
   else if (IsPIIX4Found || IsOSB4Found || IsCSB5Found)
   {
		 // added - call existing PIIX4_SMBusReadBlock function
		 return PIIX4_SMBusReadBlock (slave_addr, command_code, retval, length, 0);
   }   
   else if (IsVIAFound || IsVIA2Found)
   {
      return I2CVIAPointerSetThenReadBlock2(slave_addr, command_code, retval, length, 0);
   }
   else if (IsALIFound)
   {
      return ALI_SMBusReadBlock(slave_addr, command_code, retval, length, 0);
   }


// Unknown SMBus device
   return ERR_NOT_IMPLEMENTED;
}


///////////////////////////////////////////////////////////////////////////
// I2C H8 Read Write                                        	         //
//									                                     //
// Notes: Implements the H8 fix.                                         //
//									                                     //
// On Entry:   None.                                               	     //
//									                                     //
// On Return:  Success or failure status.                                //
//									                                     //
I2CSTATUS I2CH8ReadWrite(BYTE Address, BYTE *BufferPtr, int *Length)
{
 debug_out(DETAILS_LVL,"I2CH8ReadWrite. Start.\n");

 if (IsOSB4Found || IsCSB5Found)
     return PIIX4_H8ReadWrite (Address, BufferPtr, Length, 0);

  // Unknown SMBus device
 return (ERR_NOT_IMPLEMENTED);
}

///////////////////////////////////////////////////////////////////////////
// I2C H8 Write                                             	         //
//									                                     //
// Notes: Implements the H8 firmware fix for writes.                     //
//									                                     //
// On Entry:   None.                                               	     //
//									                                     //
// On Return:  Success or failure status.                                //
//									                                     //
I2CSTATUS I2CH8Write(BYTE Address, BYTE *BufferPtr, int Length)
{
 debug_out(DETAILS_LVL,"I2CH8Write.  Start.\n");

  if (IsOSB4Found || IsCSB5Found)
     return PIIX4_H8Write (Address, BufferPtr, Length, 0);

  // Unknown SMBus device
 return (ERR_NOT_IMPLEMENTED);
}

// Exported function that implements the Write Block protocol for SMBus.
//
// Write the value in the 'length' LSBs of retval after sending the 
// command_code to the device.
//
// slave_addr is the address of the device on the I2C bus.
// command_code is a device specific code
//
// The function returns ERR_NO_ERROR on success.
I2CSTATUS I2CWriteDataBlock(BYTE slave_addr, BYTE command_code, 
                            BYTE FAR *value, int length)
{
	if (IsICHFound)
   {
      return I2CICHPointerSetThenWriteBlock2(slave_addr, command_code, value, length, 0);
   }
	else if (IsICH2Found)
   {
      debug_out(DETAILS_LVL,"I2CWriteDataBlock.  slave_addr =%xh, command_code = %xh.\n", slave_addr, command_code);
      return I2CICHPointerSetThenWriteBlock2(slave_addr, command_code, value, length, 0);
   }
	else if (IsPIIX4Found || IsOSB4Found || IsCSB5Found)
	{
	  // added - call existing PIIX4_SMBusWriteBlock function
		return PIIX4_SMBusWriteBlock (slave_addr, command_code, value, length, 0);
	}
	else if (IsVIAFound || IsVIA2Found)
   {
      return I2CVIAPointerSetThenWriteBlock2(slave_addr, command_code, value, length, 0);
   }
    else if (IsALIFound)
   {
      return ALI_SMBusWriteBlock(slave_addr, command_code, value, length, 0);
   }

// Unknown SMBus device
   return ERR_NOT_IMPLEMENTED;
}

