
// **********************************************************************
// *                                                                    *
// * This is a common code header for the SMBUS I/O.                    *
// *                                                                    *
// * Written by the IBM Director Development Team                       *
// * (C) Copyright IBM Corporation, 1998-2003                           *
// *                                                                    *
// *  This software may be used and distributed according to the terms  *
// *  of the GNU Public License, incorporated herein by reference.      *
// *                                                                    *
// ********************************************************************** 

#ifndef I2CIO_H
#define I2CIO_H

#include "i2cos.h"

#ifdef WIN32
typedef UCHAR BYTE;
#else
typedef unsigned char BYTE;
#endif

// ------------
// Status codes
// ------------
typedef enum {
   ERR_NO_ERROR = 0,
   ERR_NO_ACK,				// 1
   ERR_TIMEOUT,				// 2
   ERR_PIIX4SMB_ERROR,		// 3
   ERR_ICHSMB_ERROR,		// 4
   ERR_VIASMB_ERROR,		// 5
   ERR_SISSMB_ERROR,		// 6
   ERR_NOT_IMPLEMENTED,		// 7
   ERR_DEVICE_TIMEOUT,		// 8
   ERR_TIMEOUT_SLAVE,		// 9
   ERR_TIMEOUT_HOST,		// 10
   ERR_COLLISION,			// 11
   ERR_HOST_KILL,			// 12
   ERR_SMBALERT,				// 13
	//next lines changed (defect 12425) to add support for
	//more PIIX4 functions on the OSB4
   ERR_FAILED,                             // 14
   ERR_BUS_ERROR,                          // 15
   ERR_DEV_ERROR,                          // 16

} I2CSTATUS;
                   
// -----------------
// Time requirements  (from I2C spec)
// -----------------
#define SETUP_TIME 10   // 1.0 us
#define PERIOD     50   // 5.0 us         

// ----------------
// GPIO definitions
// ----------------
#define DATA_HIGH_DEFAULT 1
#define CLOCK_HIGH_DEFAULT 2

//FH3: drop GPIO support
//These definitions are used in several places
//to configure a read or a write.  Removal of these definitions was
//causing the ICH module to break upon a byte read.
#define SMBus_READ_Transaction (BYTE)1
#define SMBus_WRITE_Transaction (BYTE)0


// ----------------------------
// External Function prototypes
// ----------------------------
#ifdef __OS2__
void I2CInit(int port_base_address);
#else
void I2CInit(void FAR *port_base_address);
#endif

I2CSTATUS I2CReadData(BYTE slave_addr, BYTE command_code,
                      void FAR *retval, int length);

I2CSTATUS I2CWriteData(BYTE slave_addr, BYTE command_code, 
                       void FAR *value, int length);

I2CSTATUS I2CReceive(BYTE slave_addr, void FAR *retval, int length);

I2CSTATUS I2CSend(BYTE slave_addr, void FAR *value, int length);

I2CSTATUS I2CReadDataBlock(BYTE slave_addr, BYTE command_code,
                           BYTE *retval, int *length);

I2CSTATUS I2CWriteDataBlock(BYTE slave_addr, BYTE command_code,
                            BYTE *value, int length);

I2CSTATUS I2CH8ReadWrite(BYTE Address, BYTE *BufferPtr, int *Length);

I2CSTATUS I2CH8Write(BYTE Address, BYTE *BufferPtr, int Length);

void I2CSetRetries(int num_retries);

#endif  // I2CIO_H
