
// **********************************************************************
// *                                                                    *
// * Common code file for the SMBUS on the VIA Chip                     *
// *                                                                    *
// * Written by the IBM Director Development Team                       *
// * (C) Copyright IBM Corporation, 1998-2003                           *
// *                                                                    *
// *  This software may be used and distributed according to the terms  *
// *  of the GNU Public License, incorporated herein by reference.      *
// *                                                                    *
// **********************************************************************

#ifndef I2CVIA_H
#define I2CVIA_H

// VIA PCI function 3 registers
#define VIA_SM_REG_BASE_ADDR 0x90
#define VIA_SMBHSTCFG 0xd2       //  SMBus Configuration register
#define VIA_PCICMD 0x04

#define VIA_START 0x40
#define VIA_KILL 0x02

//  VIA SMBus register offsets from base address
#define VIASMBHSTSTS 0x00       //  host status register
#define VIASMBSLVSTS 0x01       //  slave status register
#define VIASMBHSTCNT 0x02       //  host control register
#define VIASMBHSTCMD 0x03       //  host command register
#define VIASMBHSTADD 0x04       //  host address register
#define VIASMBHSTDAT0 0x05      //  host data 0 register
#define VIASMBHSTDAT1 0x06      //  host data 1 register
#define VIASMBBLKDAT  0x07      //  host block data register

// Host Status Register bit definitions
#define VIASMB_HOST_BUSY 0x1        
        
// Slave Status Register bit definitions
#define VIASMB_SLV_BUSY  0x1        

//  VIA macros
#define VIACLEAR_HOST_STATUS()  OUTP( (SMBusBaseAddr + VIASMBHSTSTS), 0x1e)
#define VIACLEAR_SLV_STATUS()  OUTP( (SMBusBaseAddr + VIASMBSLVSTS), 0x3c)
#define VIAKILL_TRANSACTION()  OUTP( (SMBusBaseAddr + VIASMBHSTCNT), 0x42)

// Exported functions
I2CSTATUS I2CVIAPointerSetThenRead2(BYTE slave_addr, BYTE command_code, void FAR *byte_read,
                                      int number_bytes_read, int number_retries);
I2CSTATUS I2CVIARead2(BYTE slave_addr, void FAR *byte_read, int number_bytes_read, int number_retries);
I2CSTATUS I2CVIAPointerSetThenWrite2(BYTE slave_addr, BYTE command_code, void FAR *bytes_written,
                                       int number_bytes_written, int number_retries);
I2CSTATUS I2CVIAPointerSetThenReadBlock2(BYTE slave_addr, BYTE command_code, BYTE FAR *byte_read,
                                    int *number_bytes_read, int number_retries);
I2CSTATUS I2CVIAPointerSetThenWriteBlock2(BYTE slave_addr, BYTE command_code, BYTE FAR *bytes_written,
                                     int number_bytes_written, int number_retries);
I2CSTATUS I2CVIAPointerSet2(BYTE address, BYTE command);

#endif  // I2CVIA_H
