
// ************************************************************************
// *                                                                      *
// * IBMI2C dll  ---- WITHOUT ----  H8 support functions                  *
// *                                                                      *
// * Portions written by the IBM Director Development Team                *
// * (C) Copyright IBM Corporation, 1998-2003                             *
// *                                                                      *
// *  This software may be used and distributed according to the terms    *
// *  of the Lesser GNU Public License, incorporated herein by reference. *
// *                                                                      *
// ************************************************************************

#include <stdio.h>
#include <stdlib.h>
#include <linux/ioctl.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <sys/ioctl.h>
#include "ibmsmb.h"
#include "dump.h"

#define DEVS 4
#define TRUE 1
#define FALSE 0
#define BYTE  unsigned char
#define UCHAR unsigned char
#define BOOL  unsigned char

#define NUMRETRIES        5
#define COMMANDCHECKSUM   0x100
#define COMMANDLENGTH     0x02


int i2c_opened = FALSE;

int ibmsmb[DEVS]; /* Device file descriptors */
int DevIdx = 0;

//-----------------------------------------------------------------------------------------------
void InitxHexxPairxs( void )  
{
  return;
} 


//-----------------------------------------------------------------------------------------------
void klog(UCHAR *bufptr, int length)
{
   Req_Buf reqbuf;  
   int iretval;
	
   DevIdx = 0;
	
   // request buf address and commmand fields not used.	
   reqbuf.length = length;
   reqbuf.data = bufptr;   
  
   iretval = ioctl(ibmsmb[DevIdx],IBMSMB_IOCTEST2, &reqbuf);
	
}  


//-----------------------------------------------------------------------------------------------
void DumpBuffer( BYTE *start, int size, BYTE *label )
/* Usage:
 *       Limit the size to dump to 25 bytes and the label to 8 characters
 */
{
  return;  
} 
	


//-----------------------------------------------------------------------------------------------
int I2C_Open()
{
   int modes[DEVS] = {  O_RDONLY, O_WRONLY, O_RDWR };
   // char *mode_names[DEVS] = { "r", "w", "rw" };
   char devname[256];

   // Force to a single device for now.
   DevIdx = 0;

   sprintf(devname,"/dev/ibmsmb%d",0);
   ibmsmb[DevIdx] = open(devname,modes[DevIdx%3]);
	
   if (ibmsmb[DevIdx] > 0) 
   {
      return TRUE;
   }
   else
   {
      return FALSE;
   }
   
}

//-----------------------------------------------------------------------------------------------
int I2C_Close()
{
   DevIdx = 0;
   //close(ibmsmb[DevIdx]);
   return TRUE;
}



//-----------------------------------------------------------------------------------------------
// Returns 0 on success, error code on failure
int I2C_SendCommandThenRead0(BYTE slave_addr, 
                             BYTE command_code, 
                             void *retval, int length)
{
      Req_Buf reqbuf;  
      int iretval;
	
      DevIdx = 0;
      reqbuf.addr = slave_addr; 
      reqbuf.command = command_code; 
      reqbuf.length = length;
      reqbuf.data = retval;   
  
      iretval = ioctl(ibmsmb[DevIdx],I2C_SEND_COMMAND_THEN_READ, &reqbuf);
      // printf("[%x] i2c SO sending I2C_SEND_COMMAND_THEN_READ IOCTL RC = %x\n",I2C_SEND_COMMAND_THEN_READ,iretval);
	
      // ioctl returns 0 for success, else a non-zero error
      return iretval;	
}



//-----------------------------------------------------------------------------------------------
// Returns 0 on success, error code on failure
int I2C_SendCommandThenRead(BYTE slave_addr, 
                            BYTE command_code, 
                            void *value, int length)
{
   int i;
   int retval;

   // Fix for the RF part -- only allows single byte reads
   for (i=0; i<length; i++)
   {
      retval = I2C_SendCommandThenRead0(slave_addr, (BYTE)(command_code+i), 
                                        ((char*)value)+i, 1);
      if (retval != 0)
         return retval;
   }
   return retval;
}



//-----------------------------------------------------------------------------------------------
int I2C_SendCommandThenWrite(BYTE slave_addr, 
                             BYTE command_code, 
                             void *value, int length)
{
      Req_Buf reqbuf;  
      int iretval;

      DevIdx = 0;
      reqbuf.addr = slave_addr; 
      reqbuf.command = command_code; 
      reqbuf.length = length;
      reqbuf.data = value;

      iretval = ioctl(ibmsmb[DevIdx], I2C_SEND_COMMAND_THEN_WRITE, &reqbuf);
      // printf("[%x] i2c SO sending I2C_SEND_COMMAND_THEN_WRITE IOCTL. RC = %x\n",I2C_SEND_COMMAND_THEN_WRITE,iretval);
	
      // ioctl returns 0 for success, else a non-zero error
      return iretval;	

}



//-----------------------------------------------------------------------------------------------
int I2C_Receive(BYTE slave_addr, 
                void *retval, int length)
{
      Req_Buf reqbuf;  
      int iretval;
	
      DevIdx = 0;
      reqbuf.addr = slave_addr; 
      reqbuf.length = length;
      reqbuf.data = retval;
	
      iretval = ioctl(ibmsmb[DevIdx], I2C_RECEIVE, &reqbuf);
      // printf("[%x] i2c SO sending I2C_RECEIVE IOCTL. RC = %x\n",I2C_RECEIVE,iretval);
	
      // ioctl returns 0 for success, else a non-zero error
      return iretval;	
	
}




//-----------------------------------------------------------------------------------------------
int I2C_Send(BYTE slave_addr, 
             void *value, int length)
{
      Req_Buf reqbuf;  
      int iretval;
	
      DevIdx = 0;
      reqbuf.addr = slave_addr; 
      reqbuf.length = length;
      reqbuf.data = value;
	
      iretval = ioctl(ibmsmb[DevIdx], I2C_SEND, &reqbuf);
      // printf("[%x] i2c SO sending I2C_SEND IOCTL. RC = %x\n",I2C_SEND,iretval);
	
      // ioctl returns 0 for success, else a non-zero error
      return iretval;	
}




//-----------------------------------------------------------------------------------------------
unsigned int I2C_Block_Read(BYTE slave_addr,
                            BYTE command_code,
                            BYTE *value,
                            int *length)
{
      Req_Buf reqbuf;  
      int iretval;
	
      DevIdx = 0;
      *length = 32;
      reqbuf.addr = slave_addr; 
      reqbuf.command = command_code;
      reqbuf.length = *length;
      reqbuf.data = value;	
	
      iretval = ioctl(ibmsmb[DevIdx], I2C_SEND_COMMAND_THEN_READ_BLOCK, &reqbuf);
      // printf("[%x] i2c SO sending I2C_SEND_COMMAND_THEN_READ_BLOCK IOCTL. RC = %x\n", I2C_SEND_COMMAND_THEN_READ_BLOCK, iretval);
      *length = reqbuf.length;  
	
      // ioctl returns 0 for success, else a non-zero error
      return iretval;
	
}




//-----------------------------------------------------------------------------------------------
unsigned int I2C_Block_Write(BYTE slave_addr,
                      BYTE command_code,
                      BYTE *value,
                      int length)
{
      Req_Buf reqbuf;  
      int iretval;
	
      DevIdx = 0;
      reqbuf.addr = slave_addr; 
      reqbuf.command = command_code;
      reqbuf.length = length;
      reqbuf.data = value;
	
      iretval = ioctl(ibmsmb[DevIdx], I2C_SEND_COMMAND_THEN_WRITE_BLOCK, &reqbuf);
      // printf("[%x] i2c SO sending I2C_SEND_COMMAND_THEN_WRITE_BLOCK IOCTL. RC = %x\n",I2C_SEND_COMMAND_THEN_WRITE_BLOCK,iretval);
	
      // ioctl returns 0 for success, else a non-zero error
      return iretval;		
	
}




//-----------------------------------------------------------------------------------------------
int I2Cdll_Init()
{
   int totRetries=0;
   //UCHAR buffer[64];
   //int retLen = 0;
   //UCHAR i;

   // Set up buffer dumping to the syslog
   InitxHexxPairxs(); 

   while (!(i2c_opened = I2C_Open()) && (totRetries++ < NUMRETRIES))
	   	sleep(0);
   if (!i2c_opened)
   {
	 return FALSE;
   }

    return TRUE;
}


/*  
//-----------------------------------------------------------------------------------------------
int main(int argc, char * argv[])
{
	int i;
	char buffer[256];

	I2C_Open();

          //  do some testing using "buffer[256]"

	I2C_Close();
}
*/
