
// ************************************************************************
// *                                                                      *
// * ibmsmb.h -- definitions for the ibmsmb module                        *
// *                                                                      *
// * Portions written by the IBM Director Development Team                *
// * (C) Copyright IBM Corporation, 1998-2003                             *
// *                                                                      *
// *  This software may be used and distributed according to the terms    *
// *  of the Lesser GNU Public License, incorporated herein by reference. *
// *                                                                      *
// ************************************************************************

/* Other Portions:
 * Copyright (C) 2001 Alessandro Rubini and Jonathan Corbet
 * Copyright (C) 2001 O'Reilly & Associates
 *
 * The source code in this file can be freely used, adapted,
 * and redistributed in source or binary form, so long as an
 * acknowledgment appears in derived source files.  The citation
 * should list that the code comes from the book "Linux Device
 * Drivers" by Alessandro Rubini and Jonathan Corbet, published
 * by O'Reilly & Associates.   No warranty is attached;
 * we cannot take responsibility for errors or fitness for use.
 */
 
#include <linux/ioctl.h>

/* version dependencies have been confined to a separate file */

//#include "sysdep.h" 


/*
 * Macros to help debugging
 */

#undef PDEBUG             /* undef it, just in case */
#ifdef IBMSMB_DEBUG
#  ifdef __KERNEL__
     /* This one if debugging is on, and kernel space */
#    define PDEBUG(fmt, args...) printk( KERN_DEBUG "ibmsmb: " fmt, ## args)
#  else
     /* This one for user space */
#    define PDEBUG(fmt, args...) fprintf(stderr, fmt, ## args)
#  endif
#else
#  define PDEBUG(fmt, args...) /* not debugging: nothing */
#endif

#undef PDEBUGG
#define PDEBUGG(fmt, args...) /* nothing: it's a placeholder */

#define IBMSMB_MAJOR 0   /* dynamic major by default */

#define IBMSMB_DEVS 4    /* ibmsmb0 through ibmsmb3 */

/*
 * The bare device is a variable-length region of memory.
 * Use a linked list of indirect blocks.
 *
 * "IbmSmb_Dev->data" points to an array of pointers, each
 * pointer refers to a memory page.
 *
 * The array (quantum-set) is IBMSMB_QSET long.
 */
#define IBMSMB_QUANTUM  4000 /* use a quantum size */
#define IBMSMB_QSET     500

#define KERNEL_DATABUF_SIZE 100


typedef struct IbmSmb_Dev {
   void **data;
   struct IbmSmb_Dev *next;  /* next listitem */
   int vmas;                 /* active mappings */
   int quantum;              /* the current allocation size */
   int qset;                 /* the current array size */
   size_t size;              /* 32-bit will suffice */
//   struct semaphore sem;     /* Mutual exclusion */
} IbmSmb_Dev;

extern IbmSmb_Dev *ibmsmb_devices;

extern struct file_operations ibmsmb_fops;

// IOCTL Request Buffer
typedef struct Req_Buf {
    unsigned char addr;
    unsigned char command;
    unsigned long length;         /* int ??? FxFx */
    void          *data;
} Req_Buf;

/*
 * The different configurable parameters
 */
extern int ibmsmb_major;     /* main.c */
extern int ibmsmb_devs;
extern int ibmsmb_order;
extern int imbsmb_qset;

/*
 * Prototypes for shared functions
 */
int ibmsmb_trim(IbmSmb_Dev *dev);
// IbmSmb_Dev *ibmsmb_follow(IbmSmb_Dev *dev, int n);



#ifndef min
#  define min(a,b) ((a)<(b) ? (a) : (b))
#endif

/*
 * Ioctl definitions
 */

/* Use 'K' as magic number */
#define IBMSMB_IOC_MAGIC  'K'

#define IBMSMB_IOCTEST1 _IOW(IBMSMB_IOC_MAGIC, 0, struct Req_Buf)

#define IBMSMB_IOCTEST2 _IOW(IBMSMB_IOC_MAGIC, 1, struct Req_Buf)  

#define I2C_RECEIVE                       _IOR(IBMSMB_IOC_MAGIC,  2, struct Req_Buf)
#define I2C_SEND_COMMAND_THEN_READ        _IOR(IBMSMB_IOC_MAGIC,  3, struct Req_Buf)
#define I2C_SEND_COMMAND_THEN_WRITE       _IOW(IBMSMB_IOC_MAGIC,  4, struct Req_Buf)
#define I2C_SEND                          _IOW(IBMSMB_IOC_MAGIC,  5, struct Req_Buf)
#define I2C_SEND_COMMAND_THEN_READ_BLOCK  _IOR(IBMSMB_IOC_MAGIC,  6, struct Req_Buf)
#define I2C_SEND_COMMAND_THEN_WRITE_BLOCK _IOW(IBMSMB_IOC_MAGIC,  7, struct Req_Buf)
#define H8_Write                          _IOW(IBMSMB_IOC_MAGIC,  8, struct Req_Buf)
#define H8_ReadWrite                      _IOWR(IBMSMB_IOC_MAGIC,  9, struct Req_Buf) 
#define GPIO_CHASSIS_CLEAR                _IO(IBMSMB_IOC_MAGIC,   10)

// .....when add, then bump the 2 lines below

#define IBMSMB_IOCDEBUGCONTROL _IO(IBMSMB_IOC_MAGIC, 11)  /* debugging control */

#define IBMSMB_IOC_MAXNR 11




