
// **********************************************************************
// *                                                                    *
// * Code for retrieving information about PCI devices.                 *
// *                                                                    *
// * Written by the IBM Director Development Team                       *
// * (C) Copyright IBM Corporation, 1998-2003                           *
// *                                                                    *
// *  This software may be used and distributed according to the terms  *
// *  of the GNU Public License, incorporated herein by reference.      *
// *                                                                    *
// **********************************************************************

#include "i2cos.h"
// #include <string.h>   FH3
// #include <stdio.h>    FH3
#include "pci.h"

char *ibm_copyright_string = "xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx.";


// Walk the PCI Bus looking for Vendor ID (VID) and Device Id (DID)
// at function 0 matching the parameters.  Returns the PCI bus and
// PCI device number of the device, if found.
// Return value:  1 if found, 0 not found
int PCIWalk(int IN vid, int IN did,
            int OUT FAR *bus, int OUT FAR *device)
{
   int i, j;
   unsigned long data;

   unsigned long match = (((unsigned long)did & 0xffff) << 16) |
                         ((unsigned long)vid & 0xffff);

   for (i=0; i<PCI_MAX_BUSES; i++)
   {
      for (j=0; j<PCI_MAX_DEVICES; j++)
      {
         data = PCI_GetConfigRegister(i, j, 0, 0);

         if (data == match)
         {
            *bus = i;
            *device = j;
            return 1;
         }
      }
   }

   return 0;
}


// target is a DoubleWord offset into the data registers (not a BYTE offset!)
unsigned long PCI_GetConfigRegister(int IN bus, int IN device,
                                    int IN fctn, int IN target)
{
   unsigned long addr;
   

   PCI_ADDR_INIT(&addr);
   PCI_ADDR_SET_BUS(&addr, bus);
   PCI_ADDR_SET_DEVICE(&addr, device);
   PCI_ADDR_SET_FCTN(&addr, fctn);
   PCI_ADDR_SET_TARGET(&addr, target);
   OUTPD(PCI_CONFIG_ADDR, addr);
   return INPD( PCI_CONFIG_DATA);
}


// target is a DoubleWord offset into the data registers (not a BYTE offset!)
void PCI_SetConfigRegister(int IN bus, int IN device,
                           int IN fctn, int IN target, 
                           unsigned long data)
{
   unsigned long addr;

   PCI_ADDR_INIT(&addr);
   PCI_ADDR_SET_BUS(&addr, bus);
   PCI_ADDR_SET_DEVICE(&addr, device);
   PCI_ADDR_SET_FCTN(&addr, fctn);
   PCI_ADDR_SET_TARGET(&addr, target);
   OUTPD(PCI_CONFIG_ADDR, addr);
   OUTPD(PCI_CONFIG_DATA, data);
   return;
}

/*
void main()
{
   int bus, device;
   int status;
   unsigned long smBaseAddr;

// Search for PIIX4 device
//   status = PCIWalk(0x8086, 0x7110, &bus, &device);
//   status = PCIWalk(0x8086, 0x1229, &bus, &device);

   if (status)
      printf("found on bus %d device %d\n", bus, device);
   else
      printf("device not found\n");

   if (status)
   {
      smBaseAddr = PCI_GetConfigRegister(bus, device, 3, 0x24);
   }
}
*/
